"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum
import httpx
from plex_api_client import utils
from plex_api_client.types import BaseModel
from plex_api_client.utils import validate_open_enum
import pydantic
from pydantic.functional_validators import PlainValidator
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class GetAllLibrariesType(str, Enum, metaclass=utils.OpenEnumMeta):
    r"""The library type"""

    MOVIE = "movie"
    TV_SHOW = "show"
    SEASON = "season"
    EPISODE = "episode"
    ARTIST = "artist"
    ALBUM = "album"


class Hidden(int, Enum):
    r"""The Plex library visibility setting"""

    VISIBLE = 0
    EXCLUDE_HOME_SCREEN = 1
    EXCLUDE_HOME_SCREEN_AND_GLOBAL_SEARCH = 2


class GetAllLibrariesLocationTypedDict(TypedDict):
    id: int
    r"""The ID of the location."""
    path: str
    r"""The path to the media item."""


class GetAllLibrariesLocation(BaseModel):
    id: int
    r"""The ID of the location."""

    path: str
    r"""The path to the media item."""


class GetAllLibrariesDirectoryTypedDict(TypedDict):
    allow_sync: bool
    r"""Indicates whether syncing is allowed."""
    art: str
    r"""URL for the background artwork of the media container."""
    composite: str
    r"""The relative path to the composite media item."""
    filters: bool
    r"""UNKNOWN"""
    refreshing: bool
    r"""Indicates whether the library is currently being refreshed or updated"""
    thumb: str
    r"""URL for the thumbnail image of the media container."""
    key: str
    r"""The library key representing the unique identifier"""
    type: GetAllLibrariesType
    title: str
    r"""The title of the library"""
    agent: str
    r"""The Plex agent used to match and retrieve media metadata."""
    scanner: str
    r"""UNKNOWN"""
    language: str
    r"""The Plex library language that has been set"""
    uuid: str
    r"""The universally unique identifier for the library."""
    updated_at: int
    r"""Unix epoch datetime in seconds"""
    scanned_at: int
    r"""Unix epoch datetime in seconds"""
    content: bool
    r"""UNKNOWN"""
    directory: bool
    r"""UNKNOWN"""
    content_changed_at: int
    r"""Timestamp (in seconds) representing the last time the content was modified.
    NOTE: Some Plex server have some absurd values for this field, like 8457612157633039800 so it should be int64

    """
    location: List[GetAllLibrariesLocationTypedDict]
    created_at: NotRequired[int]
    hidden: NotRequired[Hidden]
    r"""The Plex library visibility setting"""


class GetAllLibrariesDirectory(BaseModel):
    allow_sync: Annotated[bool, pydantic.Field(alias="allowSync")]
    r"""Indicates whether syncing is allowed."""

    art: str
    r"""URL for the background artwork of the media container."""

    composite: str
    r"""The relative path to the composite media item."""

    filters: bool
    r"""UNKNOWN"""

    refreshing: bool
    r"""Indicates whether the library is currently being refreshed or updated"""

    thumb: str
    r"""URL for the thumbnail image of the media container."""

    key: str
    r"""The library key representing the unique identifier"""

    type: Annotated[GetAllLibrariesType, PlainValidator(validate_open_enum(False))]

    title: str
    r"""The title of the library"""

    agent: str
    r"""The Plex agent used to match and retrieve media metadata."""

    scanner: str
    r"""UNKNOWN"""

    language: str
    r"""The Plex library language that has been set"""

    uuid: str
    r"""The universally unique identifier for the library."""

    updated_at: Annotated[int, pydantic.Field(alias="updatedAt")]
    r"""Unix epoch datetime in seconds"""

    scanned_at: Annotated[int, pydantic.Field(alias="scannedAt")]
    r"""Unix epoch datetime in seconds"""

    content: bool
    r"""UNKNOWN"""

    directory: bool
    r"""UNKNOWN"""

    content_changed_at: Annotated[int, pydantic.Field(alias="contentChangedAt")]
    r"""Timestamp (in seconds) representing the last time the content was modified.
    NOTE: Some Plex server have some absurd values for this field, like 8457612157633039800 so it should be int64

    """

    location: Annotated[List[GetAllLibrariesLocation], pydantic.Field(alias="Location")]

    created_at: Annotated[Optional[int], pydantic.Field(alias="createdAt")] = None

    hidden: Optional[Hidden] = Hidden.VISIBLE
    r"""The Plex library visibility setting"""


class GetAllLibrariesMediaContainerTypedDict(TypedDict):
    size: int
    r"""Number of media items returned in this response."""
    allow_sync: bool
    r"""Indicates whether syncing is allowed."""
    title1: str
    r"""The primary title of the media container."""
    directory: NotRequired[List[GetAllLibrariesDirectoryTypedDict]]


class GetAllLibrariesMediaContainer(BaseModel):
    size: int
    r"""Number of media items returned in this response."""

    allow_sync: Annotated[bool, pydantic.Field(alias="allowSync")]
    r"""Indicates whether syncing is allowed."""

    title1: str
    r"""The primary title of the media container."""

    directory: Annotated[
        Optional[List[GetAllLibrariesDirectory]], pydantic.Field(alias="Directory")
    ] = None


class GetAllLibrariesResponseBodyTypedDict(TypedDict):
    r"""The libraries available on the Server"""

    media_container: NotRequired[GetAllLibrariesMediaContainerTypedDict]


class GetAllLibrariesResponseBody(BaseModel):
    r"""The libraries available on the Server"""

    media_container: Annotated[
        Optional[GetAllLibrariesMediaContainer], pydantic.Field(alias="MediaContainer")
    ] = None


class GetAllLibrariesResponseTypedDict(TypedDict):
    content_type: str
    r"""HTTP response content type for this operation"""
    status_code: int
    r"""HTTP response status code for this operation"""
    raw_response: httpx.Response
    r"""Raw HTTP response; suitable for custom response parsing"""
    object: NotRequired[GetAllLibrariesResponseBodyTypedDict]
    r"""The libraries available on the Server"""


class GetAllLibrariesResponse(BaseModel):
    content_type: str
    r"""HTTP response content type for this operation"""

    status_code: int
    r"""HTTP response status code for this operation"""

    raw_response: httpx.Response
    r"""Raw HTTP response; suitable for custom response parsing"""

    object: Optional[GetAllLibrariesResponseBody] = None
    r"""The libraries available on the Server"""
